@@ -5,32 +5,43 @@ use thiserror::{self, Error};
 use crate::internal_prelude::*;
 use crate::{config, load_filters, status_filters, task_expanders, task_filters};
 
+pub trait ParsedDocument: 'static {}
 pub type TaskFilters<JS, TS> = Vec<Box<dyn task_filters::Filter<JS, TS> + Send + Sync>>;
 pub type StatusFilters<JS, TS> = Vec<Box<dyn status_filters::Filter<JS, TS> + Send + Sync>>;
 pub type LoadFilters<JS, TS> = Vec<Box<dyn load_filters::Filter<JS, TS> + Send + Sync>>;
-pub type TaskExpanders<JS, TS> = Vec<Box<dyn task_expanders::Expander<JS, TS> + Send + Sync>>;
+pub type TaskExpanders<JS, TS, P> = Vec<Box<dyn task_expanders::Expander<JS, TS, P> + Send + Sync>>;
+pub type DocumentParser<P> = Box<dyn Fn(Box<dyn io::Read + Sync + Send>) -> Result<P> + Send + Sync + 'static>;
 
 pub type BoxedFn<T> = Box<dyn Fn() -> Box<T> + Send + Sync>;
 pub type BoxedTaskFilter<JS, TS> = BoxedFn<dyn task_filters::Filter<JS, TS> + Send + Sync>;
 pub type BoxedStatusFilter<JS, TS> = BoxedFn<dyn status_filters::Filter<JS, TS> + Send + Sync>;
 pub type BoxedLoadFilter<JS, TS> = BoxedFn<dyn load_filters::Filter<JS, TS> + Send + Sync>;
-pub type BoxedTaskExpander<JS, TS> = BoxedFn<dyn task_expanders::Expander<JS, TS> + Send + Sync>;
+pub type BoxedTaskExpander<JS, TS, P> = BoxedFn<dyn task_expanders::Expander<JS, TS, P> + Send + Sync>;
 
-pub struct Job<JS: JobStateValues, TS: TaskStateValues> {
+pub struct Job<JS: JobStateValues, TS: TaskStateValues, P: ParsedDocument> {
 	pub url:       url::Url,
 	pub addrs:     Option<Vec<SocketAddr>>,
 	pub settings:  Arc<config::CrawlingSettings>,
-	pub rules:     Box<dyn JobRules<JS, TS>>,
+	pub rules:     Box<dyn JobRules<JS, TS, P>>,
 	pub job_state: JS,
 }
 
-impl<JS: JobStateValues, TS: TaskStateValues> Job<JS, TS> {
-	pub fn new<R: JobRules<JS, TS>>(
+impl<JS: JobStateValues, TS: TaskStateValues, P: ParsedDocument> Job<JS, TS, P> {
+	pub fn new<R: JobRules<JS, TS, P>>(
+		url: &str,
+		settings: config::CrawlingSettings,
+		rules: R,
+		job_state: JS,
+	) -> anyhow::Result<Job<JS, TS, P>> {
+		Self::new_with_shared_settings(url, Arc::new(settings), rules, job_state)
+	}
+
+	pub fn new_with_shared_settings<R: JobRules<JS, TS, P>>(
 		url: &str,
 		settings: Arc<config::CrawlingSettings>,
 		rules: R,
 		job_state: JS,
-	) -> anyhow::Result<Job<JS, TS>> {
+	) -> anyhow::Result<Job<JS, TS, P>> {
 		let url = Url::parse(url).context("cannot parse url")?;
 
 		Ok(Self { url, addrs: None, settings, rules: Box::new(rules), job_state })
@@ -42,17 +53,18 @@ impl<JS: JobStateValues, TS: TaskStateValues> Job<JS, TS> {
 	}
 }
 
-#[derive(Clone)]
-pub struct ResolvedJob<JS: JobStateValues, TS: TaskStateValues> {
+#[derive(Derivative)]
+#[derivative(Clone(bound = ""))]
+pub struct ResolvedJob<JS: JobStateValues, TS: TaskStateValues, P: ParsedDocument> {
 	pub url:      url::Url,
 	pub addrs:    Option<Vec<SocketAddr>>,
 	pub settings: Arc<config::CrawlingSettings>,
-	pub rules:    Arc<Box<dyn JobRules<JS, TS>>>,
+	pub rules:    Arc<Box<dyn JobRules<JS, TS, P>>>,
 	pub ctx:      JobCtx<JS, TS>,
 }
 
-impl<JS: JobStateValues, TS: TaskStateValues> From<Job<JS, TS>> for ResolvedJob<JS, TS> {
-	fn from(job: Job<JS, TS>) -> ResolvedJob<JS, TS> {
+impl<JS: JobStateValues, TS: TaskStateValues, P: ParsedDocument> From<Job<JS, TS, P>> for ResolvedJob<JS, TS, P> {
+	fn from(job: Job<JS, TS, P>) -> ResolvedJob<JS, TS, P> {
 		ResolvedJob {
 			url:      job.url.clone(),
 			addrs:    job.addrs,
@@ -63,14 +75,15 @@ impl<JS: JobStateValues, TS: TaskStateValues> From<Job<JS, TS>> for ResolvedJob<
 	}
 }
 
-pub trait JobRules<JS: JobStateValues, TS: TaskStateValues>: Send + Sync + 'static {
+pub trait JobRules<JS: JobStateValues, TS: TaskStateValues, P: ParsedDocument>: Send + Sync + 'static {
 	fn task_filters(&self) -> TaskFilters<JS, TS>;
 	fn status_filters(&self) -> StatusFilters<JS, TS>;
 	fn load_filters(&self) -> LoadFilters<JS, TS>;
-	fn task_expanders(&self) -> TaskExpanders<JS, TS>;
+	fn task_expanders(&self) -> TaskExpanders<JS, TS, P>;
+	fn document_parser(&self) -> Arc<DocumentParser<P>>;
 }
 
-pub type BoxedJobRules<JS, TS> = Box<dyn JobRules<JS, TS>>;
+pub type BoxedJobRules<JS, TS, P> = Box<dyn JobRules<JS, TS, P>>;
 
 #[derive(Error, Debug)]
 pub enum Error {
@@ -79,44 +92,44 @@ pub enum Error {
 	#[error("timeout during loading")]
 	LoadTimeout,
 	#[error("terminated by status filter {name:?}")]
-	StatusFilterTerm { name: String },
+	StatusFilterTerm { name: &'static str },
 	#[error("terminated by error in status filter(term_by_error=true) {name:?}")]
 	StatusFilterTermByError {
-		name:   String,
+		name:   &'static str,
 		#[source]
 		source: anyhow::Error,
 	},
 	#[error("terminated by panic in status filter {name:?}")]
 	StatusFilterTermByPanic {
-		name:   String,
+		name:   &'static str,
 		#[source]
 		source: anyhow::Error,
 	},
 	#[error("terminated by load filter {name:?}")]
-	LoadFilterTerm { name: String },
+	LoadFilterTerm { name: &'static str },
 	#[error("terminated by error in load filter(term_by_error=true) {name:?}")]
 	LoadFilterTermByError {
-		name:   String,
+		name:   &'static str,
 		#[source]
 		source: anyhow::Error,
 	},
 	#[error("terminated by panic in load filter {name:?}")]
 	LoadFilterTermByPanic {
-		name:   String,
+		name:   &'static str,
 		#[source]
 		source: anyhow::Error,
 	},
 	#[error("terminated by task expander {name:?}")]
-	TaskExpanderTerm { name: String },
+	TaskExpanderTerm { name: &'static str },
 	#[error("terminated by error in task expander(term_by_error=true) {name:?}")]
 	TaskExpanderTermByError {
-		name:   String,
+		name:   &'static str,
 		#[source]
 		source: anyhow::Error,
 	},
 	#[error("terminated by panic in task expander {name:?}")]
 	TaskExpanderTermByPanic {
-		name:   String,
+		name:   &'static str,
 		#[source]
 		source: anyhow::Error,
 	},
@@ -294,9 +307,9 @@ pub struct ParserResponse {
 	pub work_duration: Duration,
 }
 
-pub trait JobStateValues: Send + Sync + Clone + 'static {}
+pub trait JobStateValues: Send + Sync + 'static {}
 
-impl<T: Send + Sync + Clone + 'static> JobStateValues for T {}
+impl<T: Send + Sync + 'static> JobStateValues for T {}
 
 pub trait TaskStateValues: Send + Sync + Clone + Default + 'static {}
 
@@ -304,7 +317,8 @@ impl<T: Send + Sync + Clone + Default + 'static> TaskStateValues for T {}
 
 pub type JobSharedState = Arc<Mutex<HashMap<String, Box<dyn std::any::Any + Send + Sync>>>>;
 
-#[derive(Clone)]
+#[derive(Derivative)]
+#[derivative(Clone(bound = "TS: Clone"))]
 pub struct JobCtx<JS, TS> {
 	pub settings:   Arc<config::CrawlingSettings>,
 	pub started_at: Instant,
@@ -353,7 +367,7 @@ impl<JS: JobStateValues, TS: TaskStateValues> JobCtx<JS, TS> {
 }
 
 impl Link {
-	pub(crate) fn new(
+	pub fn new(
 		href: &str,
 		rel: &str,
 		alt: &str,
@@ -362,14 +376,12 @@ impl Link {
 		target: LinkTarget,
 		parent: &Link,
 	) -> Result<Self> {
-		let link_str = href.splitn(2, '#').next().context("cannot prepare link")?;
-
-		let url = Url::parse(link_str).unwrap_or(
-			parent
-				.url
-				.join(link_str)
-				.with_context(|| format!("cannot join relative href {} to {}", href, &parent.url))?,
-		);
+		let mut url = Url::parse(href)
+			.or_else(|_err| {
+				parent.url.join(href).with_context(|| format!("cannot join relative href {} to {}", href, &parent.url))
+			})
+			.context("cannot parse url")?;
+		url.set_fragment(None);
 
 		Ok(Self {
 			url,
